#include "rszvl.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rszvl_S-parameter_measurement_example.h"


/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else

#define ARRAY_LEN 5000

/* Global variables */
static   ViInt32    panelHandle  = 0,
                    traceDisplay;
static   ViInt16    sweepNumber,
                    paramOut   	 = 2,
                    paramIn 	 = 1;
volatile ViBoolean  enableTrace  = VI_FALSE;
static   ViSession  instrSession = VI_NULL;
         ViReal64   amplitude [ARRAY_LEN],
			 		real[ARRAY_LEN/2],
					imag[ARRAY_LEN/2],
					startFreq	= 300.0e3,
					stopFreq	= 4.0e9;
    


/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void    cleanup (ViBoolean deviceClear);
void    EnableButtons(ViBoolean enable);



int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rszvl_S-parameter_measurement_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

/*===========================================================================*/
/* Function: Exit Button Callback
/* Purpose:  Closes program window
/*===========================================================================*/
int  CVICALLBACK exitButton(int panel, int control, int event, 
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            enableTrace = VI_FALSE;
            if (instrSession != 0)
            {
                rszvl_close (instrSession);
                instrSession = 0;
            }
            QuitUserInterface (0);
            
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument according to selected measurement
/*           parameters and enables the measurement loop
/*===========================================================================*/
int  CVICALLBACK startButton(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [256] = "",
                catalogString [256] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
            {
            
                /* block mouse events */
                SetWaitCursor (1);
                
                /* read config from panel*/
                GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
                GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
                GetCtrlVal (panelHandle, PANEL_RESET, &reset);
                GetCtrlVal (panelHandle, PANEL_DISPLAY, &display);
                GetCtrlVal (panelHandle, PANEL_SWEEP_NUMBER, &sweepNumber);
                

                /* disable buttons for configuration */
                EnableButtons(VI_FALSE);
                
                /* initialize driver */
                if (instrSession == VI_NULL )
                    CHECKERR (rszvl_init (instr_desc, id, reset, &instrSession ));
                else if (reset == VI_TRUE ) /* reset device box checked */
                    CHECKERR (rszvl_reset (instrSession));
				
				/* delete default trace (after reset) */
				if (reset == VI_TRUE)
				{
					CHECKERR( rszvl_TraceDelete (instrSession, 1, "Trc1"));
				}
                
                /* reduce number of sweep points to increase speed of configuration*/
                CHECKERR(rszvl_ConfigureSweepPoints(instrSession,1,2));
                
                /* switch device display on/off */
                CHECKERR(rszvl_ConfigureDisplayUpdate (instrSession, display, VI_FALSE));
 
                /* Set instrument to single sweep mode - continous mode is done
                 * programmatically because this is the only way how to
                 * synchronize measurement using *OPC
                 */
                CHECKERR (rszvl_ConfigureAcquisition (instrSession, VI_FALSE, 1));
    			
                /* read trace parameters from table */
                GetCtrlVal (panelHandle, PANEL_S_PARAMETER_OUT, &paramOut);
                GetCtrlVal (panelHandle, PANEL_S_PARAMETER_IN, &paramIn);
                GetCtrlVal (panelHandle, PANEL_START_FREQUENCY, &startFreq);
                GetCtrlVal (panelHandle, PANEL_STOP_FREQUENCY, &stopFreq);
					
                /* Add channel */
                CHECKERR (rszvl_ChannelAdd (instrSession, 1, "Ch1"));
                        
                /* List all existing traces */   
                CHECKERR (rszvl_TraceList (instrSession, 1, 256, catalogString));
                
				/* Create noexisting trace */
                if(NULL == strstr (catalogString, "Trace_1"))
				{
                    CHECKERR (rszvl_TraceAddDiagramArea (instrSession, 1, 1, 1, "Trace_1"));
                }                        
                	
				/* Set S-parameter */
                CHECKERR(rszvl_ConfigureSParameters (instrSession, 1, "Trace_1", paramOut, paramIn));
				
				/* Set Start and Stop frequency */
				CHECKERR(rszvl_ConfigureFrequencyStartStop (instrSession, startFreq, stopFreq));
                
                /* set number of sweep points */
                CHECKERR (rszvl_ConfigureSweepPoints (instrSession, 1, sweepNumber));
                
                /* start measurements */
                enableTrace = VI_TRUE;
                
                /* enable mouse events*/
                SetWaitCursor (0);
            }
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Stop Button Callback
/* Purpose:  Stops the continuous measurement, in other cases has no sense
/*===========================================================================*/
int CVICALLBACK stopButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            
            /* stop measurements */
            enableTrace = VI_FALSE;
            
            /* enable buttons for configuration */
            EnableButtons(VI_TRUE);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Timer Callback
/* Purpose:  Every timer tick the traces are read out of the device and plotted
/*           as a graph. Measurements has to be enabled otherwise callback
/*           returns immediatelly
/*===========================================================================*/
int  CVICALLBACK timer(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViStatus    temp_status  = VI_SUCCESS,
                status       = VI_SUCCESS;
    ViChar      tempString [10]  = "";
    ViInt32     numberOfValues = 0;
    int         i,j;
    
    switch (event)
    {
        case EVENT_TIMER_TICK:  /* every 100ms */
            
            /* already configured and ready for measurements?*/
            if(enableTrace==VI_FALSE)
                return 0;
            
            /* exclusive execution from now on! */          
            enableTrace = VI_FALSE;
            
            /* initiate measurement */ 
            CHECKERR (rszvl_SANInitiate (instrSession, 15000));
                        
            DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
		                
            /*read single sweep data*/
            CHECKERR (rszvl_GetTraceResponseSingleSweepData (instrSession, 1, 1,
                                                             &numberOfValues, amplitude));
                        
			/*split real and imag part for plotting*/
            for(j=0;j<ARRAY_LEN;j+=2)
            {
				
                real[j/2]=amplitude[j];
                imag[j/2]=amplitude[j+1];
            }
                        
            /* Plots a waveform onto a graph control */
			temp_status = PlotWaveform (panelHandle, PANEL_GRAPH, real,
										numberOfValues/2, VAL_DOUBLE, 1.0, 0.0,
										0.0, 1, VAL_THIN_LINE, VAL_SIMPLE_DOT,
										VAL_SOLID, 1, VAL_WHITE);
                        
            if( temp_status < 0 )
            {
                /* prompt CVI error message */
                MessagePopup ("Error", GetUILErrorString(temp_status));
                
                /* cleanup zvl driver session */
                cleanup(VI_FALSE);
                
                /* do not free resource */
                enableTrace = VI_FALSE;
                return -1;
            }
                        
            /* Plots a waveform onto a graph control */
			temp_status = PlotWaveform (panelHandle, PANEL_GRAPH, imag,
										numberOfValues/2, VAL_DOUBLE, 1.0, 0.0,
										0.0, 1, VAL_THIN_LINE, VAL_SIMPLE_DOT,
										VAL_SOLID, 1, VAL_YELLOW);
            
			if( temp_status < 0 )
            {
                /* prompt CVI error message */
                MessagePopup ("Error", GetUILErrorString(temp_status));
                
                /* cleanup zvl driver session */
                cleanup(VI_FALSE);
                
                /* do not free resource */
                enableTrace = VI_FALSE;
                return -1;
            }
            
            /* free resource */         
            enableTrace = VI_TRUE;
            
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [256];
    ViChar  error_buffer [1024];
    ViChar*  p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rszvl_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        /* This function reads an error code and a message from the instrument's error queue */
        rszvl_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }

        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rszvl_close (instrSession);
    instrSession = VI_NULL;
    
    /* no measurements */
    enableTrace  = VI_FALSE;
    
    /* enable GUI */
    EnableButtons (1);
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_S_PARAMETER_OUT, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_S_PARAMETER_IN,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_START,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_DISPLAY,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_SWEEP_NUMBER, 	  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_START_FREQUENCY, ATTR_DIMMED, !enable);
	SetCtrlAttribute (panelHandle, PANEL_STOP_FREQUENCY,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP,       ATTR_DIMMED,  enable);
    
}
